﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ImageViewer
{
    public partial class ImageView : Form
    {
        private string _filename;

        private int _imageIndex;

        private MainForm _parentMainForm;

        private string _basicInfo;

        private Bitmap _bitmap;

        private float _ratio = 0;
        

        public ImageView(MainForm parentMainForm, string fileName, Image image, string info, int imageIndex)
        {
            InitializeComponent();

            _parentMainForm = parentMainForm;
            _filename = fileName;
            _bitmap = new Bitmap(image);
            pbImage.Image = image;
            _basicInfo = info;
            _imageIndex = imageIndex;
            RewriteInfo();
        }

        private void RewriteInfo()
        {
            RecalcBasicInfo();
            string ratio = "Optimal";
            tssInfo.Text = string.Format("{0}, Size({1}), image {2}/{3}", new string[4] { _basicInfo, ratio, (_imageIndex + 1).ToString(), _parentMainForm.CurrentImages.Length.ToString()});
        }

        private void ImageView_KeyDown(object sender, KeyEventArgs e)
        {
            switch(e.KeyCode)
            {
                case Keys.Escape:
                    _parentMainForm.Close();
                    break;
                case Keys.Enter:
                    Exit();
                    break;
                case Keys.Home:
                    GotoFirst();
                    break;
                case Keys.End:
                    GotoLast();
                    break;
                case Keys.PageUp:
                    GotoPrev();
                    break;
                case Keys.PageDown:
                    GotoNext();
                    break;
                case Keys.H:
                    MirrorHorizontal();
                    break;
                case Keys.V:
                    MirrorVertical();
                    break;
                case Keys.L:
                    RotateLeft();
                    break;
                case Keys.R:
                    RotateRight();
                    break;
                case Keys.OemBackslash:
                    pbImage.SizeMode = PictureBoxSizeMode.Zoom;
                    break;
            }
            
            //else if (e.KeyData == Keys.)
            //    pbImage.SizeMode = PictureBoxSizeMode.AutoSize;
        }

        private void Exit()
        {
            this.Close();
        }

        private void GotoFirst()
        {
            _filename = _parentMainForm.CurrentImages[0];
            pbImage.Image = _parentMainForm.GetImage(0);
            _imageIndex = 0;
            RewriteInfo();
        }

        private void GotoLast()
        {
            _filename = _parentMainForm.CurrentImages[_parentMainForm.CurrentImages.Length - 1];
            pbImage.Image = _parentMainForm.GetImage(_parentMainForm.CurrentImages.Length - 1);
            _imageIndex = _parentMainForm.CurrentImages.Length - 1;
            RewriteInfo();
        }

        private void GotoNext()
        {
            _imageIndex++;
            if (_imageIndex < _parentMainForm.CurrentImages.Length)
            {
                _filename = _parentMainForm.CurrentImages[_imageIndex];
                pbImage.Image = _parentMainForm.GetImage(_imageIndex);
                RewriteInfo();
            }
            else
            {
                _imageIndex--;
            }
        }

        private void GotoPrev()
        {
            _imageIndex--;
            if (_imageIndex >= 0)
            {
                _filename = _parentMainForm.CurrentImages[_imageIndex];
                pbImage.Image = _parentMainForm.GetImage(_imageIndex);
                RewriteInfo();
            }
            else
            {
                _imageIndex++;
            }
        }

        private void miGotoFirst_Click(object sender, EventArgs e)
        {
            GotoFirst();
        }

        private void miGoToLast_Click(object sender, EventArgs e)
        {
            GotoLast();
        }

        private void RecalcBasicInfo()
        {
            FileStream fs = new FileStream(_filename, FileMode.Open);
            long size = fs.Length / (long)1024;
            fs.Close();
            
            FileInfo finfo = new FileInfo(_filename);
            string dateTime = finfo.CreationTime.ToString();
            string width = pbImage.Image.PhysicalDimension.Width.ToString();
            string height = pbImage.Image.PhysicalDimension.Height.ToString();

            _basicInfo = string.Format("{0} : {1}KB", Path.GetFileName(_filename), size) + ", " + string.Format("{0}, {1}x{2}", new string[3] { dateTime, width, height });
        }

        private void miGoToPrev_Click(object sender, EventArgs e)
        {
            GotoPrev();
        }

        private void miGotoNext_Click(object sender, EventArgs e)
        {
            GotoNext();
        }

        private void pbImage_DoubleClick(object sender, EventArgs e)
        {
            Exit();
        }

        private void horizontalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MirrorHorizontal();
        }

        private void MirrorHorizontal()
        {
            pbImage.Image.RotateFlip(RotateFlipType.RotateNoneFlipX);
            pbImage.Refresh();
        }

        private void MirrorVertical()
        {
            pbImage.Image.RotateFlip(RotateFlipType.RotateNoneFlipY);
            pbImage.Refresh();
        }

        private void verticalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MirrorVertical();
        }

        private void RotateLeft()
        {
            pbImage.Image.RotateFlip(RotateFlipType.Rotate90FlipNone);
            pbImage.Refresh();
        }

        private void RotateRight()
        {
            pbImage.Image.RotateFlip(RotateFlipType.Rotate270FlipNone);
            pbImage.Refresh();
        }

        private void leftToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RotateLeft();
        }

        private void rightToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RotateRight();
        }

        private void PreloadImages()
        {
            // Images is taken from Preview on MainForm, so they're still preloaded
        }

        private void grayscaleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Grayscale();
        }

        private void Grayscale()
        {
            Graphics g = Graphics.FromImage(pbImage.Image);
            Bitmap b = new Bitmap(pbImage.Image);

            for (int y = 0; y < b.Height; y++)
            {
                for (int x = 0; x < b.Width; x++)
                {
                    Color c = b.GetPixel(x, y);
                    int col = (int)((float)c.GetBrightness() * 100.0f);
                    Color dest = Color.FromArgb(col, col, col);
                    b.SetPixel(x, y, dest);
                }
            }
            pbImage.Image = b;
            pbImage.Refresh();
        }

        private void miExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void toolStripMenuItem2_Click(object sender, EventArgs e)
        {
            ZoomIn();
        }

        private void ZoomIn()
        {
            if (_ratio == 0)
                _ratio = 0.9f;
            else
                _ratio -= 0.1f;
            if (_ratio <= 0) _ratio = 0.1f;

            // TODO Here
            //pbImage.SizeMode = PictureBoxSizeMode.CenterImage;
            //Bitmap b = new Bitmap(_bitmap.Width * _ratio, _bitmap.Height * _ratio);
            //pbImage.Image = _bitmap;
            //Graphics g = Graphics.FromImage(pbImage.Image);
            //g.DrawImage(_bitmap, 0, 0, _bitmap.Width * _ratio, _bitmap.Height * _ratio);
            //pbImage.Refresh();

        }

        private void outToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ZoomOut();
        }

        private void ZoomOut()
        {
            
        }
    }
}
